//
// Standard Functions
//
float round(float x)
{ return sign(x)*floor(abs(x)+0.5); }
//
// Spatial Dithering
//
float Bayerdithering2(vec2 xy)
{
	mat2 dithermatrix = mat2(1.0,3.0,4.0,2.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering4(vec2 xy)
{
	mat4 dithermatrix = mat4(1.0,9.0,3.0,11.0,15.0,5.0,13.0,7.0,4.0,12.0,2.0,10.0,14.0,8.0,16.0,6.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering8(vec2 xy)
{
	float limit;
	int x = int(xy.x);
	int y = int(xy.y);
	
	mat4 dithermatrix1 = mat4(1.0,33.0,57.0,9.0,49.0,17.0,25.0,41.0,47.0,31.0,21.0,45.0,15.0,63.0,37.0,5.0);
	mat4 dithermatrix2 = mat4(3.0,35.0,43.0,11.0,51.0,19.0,27.0,59.0,53.0,29.0,23.0,39.0,13.0,61.0,55.0,7.0);
	mat4 dithermatrix3 = mat4(4.0,52.0,60.0,12.0,36.0,20.0,28.0,44.0,46.0,30.0,24.0,56.0,14.0,62.0,40.0,8.0);
	mat4 dithermatrix4 = mat4(2.0,50.0,42.0,10.0,34.0,18.0,26.0,58.0,48.0,32.0,22.0,54.0,16.0,64.0,38.0,6.0);

	if(x<4) // dithermatrix1 or dithermatrix3
	{
		if(y<4) limit = dithermatrix1[x][y];
		else limit = dithermatrix3[x][y-4];
	}
	else // dithermatrix2 or dithermatrix4
	{
		if(y<4) limit = dithermatrix2[x-4][y];
		else limit = dithermatrix4[x-4][y-4];
	}
	return limit;
}
//
// CARTESIAN CHECKERBOARD STIMULUS
// Assume:
// RingRowSize, ColumnWedgeSize in pixels
// RXPhase, ThetaYPhase, RXShift, ThetaYShift in radians
vec2 CartesianCheckerElement( float x, float y,
						float NbRowRings, float NbColumnWedges, float RingRowSize, float ColumnWedgeSize,	// Shape Parameters
						float RXPhase, float ThetaYPhase, float RXShift, float ThetaYShift,					// Phase/Shift Parameters
						float discRXShift, float discThetaYShift, 
						float RXAxisModulation, float ThetaYAxisModulation)									// Options
{
	// Spatial frequency & radius
	float sfrad1 = pi/RingRowSize;
	float sfrad2 = pi/ColumnWedgeSize;
	float sizeradius1 = NbRowRings*RingRowSize/2.;
	float sizeradius2 = NbColumnWedges*ColumnWedgeSize/2.;
	//
	// Compute Carriers
	//
	//float caror = 0.;
	//float orad = M_PI*(90.+caror)/180.;	// Make sure the orientation indicates the "bars" orientation (ie 0 for horizontal, 90 for vertical)
	//float sinorad = sin(orad);
	//float cosorad = cos(orad);

	// Phase info (converted to radians) already converted
	// RXPhase = pi*RXPhase/180.;
	// ThetaYPhase = pi*ThetaYPhase/180.;
	float phaserad1, phaserad2;
	if(NbRowRings!=1.) phaserad1 = ThetaYPhase;
	else phaserad1 = ThetaYPhase>0.?pi/2.:-pi/2.;
	// Note sure whether/why testing !=1 below is necessary because this does not work for 1 wedge with 360 deg,
	// Workaround: Use 2 wedges at 361 deg for pure concentric (see now singular cases above)
	if(NbColumnWedges!=1.) phaserad2 = RXPhase;
	else phaserad2 = RXPhase>0.?pi/2.:-pi/2.;

	// Shift Info (converted to radians per pixel) Conversion already done
	// RXShift = pi*RXShift/180.;
	// ThetaYShift = pi*ThetaYShift/180.;
			
	// Embedded rotation disable to prevent horizontal/vertical artefact (but the 90 deg rotation still applied!)
	// rotatedx = y; //cosorad * x + sinorad * y;
	// rotatedy = -x;//- sinorad * x + cosorad * y;
	float tmpx = sfrad1*y+phaserad1;
	float tmpy = -sfrad2*x+phaserad2;
	float carrier1, carrier2;
	if(discThetaYShift!=0.) carrier2 = sin(tmpx+round(0.5+tmpy/pi)*ThetaYShift);
	else carrier2 = sin(tmpx+ThetaYShift*x);
	if(discRXShift!=0.) carrier1 = sin(tmpy+round(0.5+tmpx/pi)*RXShift);
	else carrier1 = sin(tmpy+RXShift*y);
	
	// Binarize each carrier to {-1,1} if square-wave
	if(RXAxisModulation==1.) carrier1 = sign(carrier1); // modulationSQUAREWAVE
	if(ThetaYAxisModulation==1.) carrier2 = sign(carrier2); // modulationSQUAREWAVE
	
	// Apply envelope to each carrier as function of the checker type
	// Set Alpha channel as envelope as well
	// Rectangular Envelope for cartesian coordinates
	float alpha = step(abs(x),sizeradius2)*step(abs(y),sizeradius1);

	// Compute the carriers combination
	float envelop = carrier1*carrier2*alpha;

	return vec2(envelop, alpha);
}
//
// POLAR CHECKERBOARD STIMULUS
// Assume:
// ColumnWedgePos in radians
// RXPhase, ThetaYPhase, RXShift, ThetaYShift in radians
// ColumnWedgeSize in unitless (in deg but converted to radian here)
vec2 PolarCheckerElement( float r, float theta,
						float RingRowPos, float ColumnWedgePos,												// Position Shift
						float NbRowRings, float NbColumnWedges, float RingRowSize, float ColumnWedgeSize,	// Shape Parameters
						float RXPhase, float ThetaYPhase, float RXShift, float ThetaYShift,					// Phase/Shift Parameters
						float discRXShift, float discThetaYShift, 
						float RXAxisModulation, float ThetaYAxisModulation)				// Options
{
	//
	// Deal with singular cases:
	//
	// Purely concentric pattern
	// if(CheckerType==checkerPOLAR && NbColumnWedges==1 && ColumnWedgeSize==360.f) ColumnWedgeSize = 360.01f; // Prevent the horizontsl line artefact

	// Spatial frequency & radius
	float sfrad1 = pi/RingRowSize;
	float sf = round(360./(ColumnWedgeSize*2.));	// Make sure to take integer only for Angular pattern (nb cycles per circumference) round added on GLSL
	float sizeradius1 = NbRowRings*RingRowSize;
	float sizeradius2 = NbColumnWedges*ColumnWedgeSize/2.;
	sizeradius2 = min(sizeradius2,360.);	// checkerPOLAR: Make sure that polar size is not more than one full circomference

	//  Assume Wedge position in rads in [-180,180] range
	float anglemax = sizeradius2*pi/180.;
	float wedgeposrad = ColumnWedgePos;
	
	//
	// Compute Carriers
	//
	//float caror = 0.;
	//float orad = M_PI*(90.+caror)/180.;	// Make sure the orientation indicates the "bars" orientation (ie 0 for horizontal, 90 for vertical)
	//float sinorad = sin(orad);
	//float cosorad = cos(orad);

	// Phase info (converted to radians) already converted
	// RXPhase = pi*RXPhase/180.;
	// ThetaYPhase = pi*ThetaYPhase/180.;
	float phaserad1, phaserad2;
	if(NbRowRings!=1.) phaserad1 = ThetaYPhase;
	else phaserad1 = ThetaYPhase>0.?pi/2.:-pi/2.;
	// Note sure whether/why testing !=1 below is necessary because this does not work for 1 wedge with 360 deg,
	// Workaround: Use 2 wedges at 361 deg for pure concentric (see now singular cases above)
	if(NbColumnWedges!=1.) phaserad2 = RXPhase;
	else phaserad2 = RXPhase>0.?pi/2.:-pi/2.;

	// Shift Info (converted to radians per pixel) already converted
	// RXShift = pi*RXShift/180.;
	// ThetaYShift = pi*ThetaYShift/180.;
	float tmpR = sfrad1*r+phaserad2;
	float tmpT = sf*(theta-wedgeposrad)+phaserad1;
	float carrier1, carrier2;
	if(discRXShift!=0.) carrier1 = sin(tmpR + round(0.5+tmpT/pi)*RXShift); // jumpy (sync on angular sampling) or smooth R phase shift
	else carrier1 = sin(tmpR + RXShift*theta); // jumpy (sync on angular sampling) or smooth R phase shift
	if(discThetaYShift!=0.) carrier2 = sin(tmpT + round(0.5+tmpR/pi)*ThetaYShift); // sf*(*(angptr++)-wedgeposrad)+phaserad1;
	else carrier2 = sin(tmpT + ThetaYShift*r); // sf*(*(angptr++)-wedgeposrad)+phaserad1;
	
	// Binarize each carrier to {-1,1} if square-wave
	if(RXAxisModulation==1.) carrier1 = sign(carrier1); // modulationSQUAREWAVE
	if(ThetaYAxisModulation==1.) carrier2 = sign(carrier2); // modulationSQUAREWAVE
	
	// Apply envelope to each carrier as function of the checker type
	// Set Alpha channel as envelope as well
	// Wedge Envelope for polar coordinates
	float minradius = RingRowPos;
	float maxradius = RingRowPos+sizeradius1;
	float tmpval = abs( theta-wedgeposrad);
	if(tmpval>pi) tmpval = 2.*pi-tmpval;
	float alpha = step(minradius,r)*step(r,maxradius)*step(tmpval,anglemax);
	
	// Compute the carriers combination
	float envelop = carrier1*carrier2*alpha;

	return vec2(envelop, alpha);
}
