//
// Standard Functions
//
float round(float x)
{ return sign(x)*floor(abs(x)+0.5); }
float rect(float x) { if(x>0.) return x; else return 0.; }
float log10(float x) { return log(x)/log(10.); }
float dist( float x, float y) { return sqrt(x*x+y*y); }
float hypot( float x, float y) { return sqrt(x*x+y*y); }
float atan2( float y, float x) { return atan(y,x); }
float deg(float x) { return degrees(x); }
float rad(float x) { return radians(x); }
// Note: the original GLSL smoothstep has x arg as last arg which is not consistent with Psykinematix's expression syntax
float hsmoothstep(float x, float edge, float borderextent) { return smoothstep(edge, edge+borderextent, x); }
float cminmax(float x, float min, float max) { return clamp(x, min, max); } // min(max(x,min),max)
float clip(float x, float min, float max) { return clamp(x, min, max); }
float cmin(float x, float min) { return max(x,min); }
float cmax(float x, float max) { return min(x,max); }
//
// Spatial Dithering
//
float Bayerdithering2(vec2 xy)
{
	mat2 dithermatrix = mat2(1.0,3.0,4.0,2.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering4(vec2 xy)
{
	mat4 dithermatrix = mat4(1.0,9.0,3.0,11.0,15.0,5.0,13.0,7.0,4.0,12.0,2.0,10.0,14.0,8.0,16.0,6.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering8(vec2 xy)
{
	float limit;
	int x = int(xy.x);
	int y = int(xy.y);
	
	mat4 dithermatrix1 = mat4(1.0,33.0,57.0,9.0,49.0,17.0,25.0,41.0,47.0,31.0,21.0,45.0,15.0,63.0,37.0,5.0);
	mat4 dithermatrix2 = mat4(3.0,35.0,43.0,11.0,51.0,19.0,27.0,59.0,53.0,29.0,23.0,39.0,13.0,61.0,55.0,7.0);
	mat4 dithermatrix3 = mat4(4.0,52.0,60.0,12.0,36.0,20.0,28.0,44.0,46.0,30.0,24.0,56.0,14.0,62.0,40.0,8.0);
	mat4 dithermatrix4 = mat4(2.0,50.0,42.0,10.0,34.0,18.0,26.0,58.0,48.0,32.0,22.0,54.0,16.0,64.0,38.0,6.0);

	if(x<4) // dithermatrix1 or dithermatrix3
	{
		if(y<4) limit = dithermatrix1[x][y];
		else limit = dithermatrix3[x][y-4];
	}
	else // dithermatrix2 or dithermatrix4
	{
		if(y<4) limit = dithermatrix2[x-4][y];
		else limit = dithermatrix4[x-4][y-4];
	}
	return limit;
}
//
// Noise Functions
//
// Differences with original version: addition of seed parameter
float unoise_rand(vec2 co, float seed) // returns a pseudo-random number in [0,1] range 
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * (43758.5453+seed));
	// original without see arg: return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}
// Not used: float noise(float x, float y, float seed) { vec2 co = vec2(x,y); return rand(co, seed); }
//
//	Uniform Noise:	unoise(x, seed, granularity)
//
// Note: using sampled x to the seed prevent the occurences of diagonal patterns with Intel HD Graphics 4000
float unoise(float x, float seed, float granularity) // Psykinematix specification
{
	vec2 sampling = float(granularity)*floor(coordVar/granularity);
	float xseed = float(granularity)*floor(x/granularity);
	return 2.*unoise_rand(sampling,seed+xseed)-1.; // [-1,1] range
	// return rand(fract(coordVar*float(granularity))/size);
}
//
//	Perlin Noise: pnoise(x,y)
//
vec3 pnoise_mod289(vec3 x) {return x - floor(x * (1.0 / 289.0)) * 289.0;}
vec2 pnoise_mod289(vec2 x) {return x - floor(x * (1.0 / 289.0)) * 289.0;}
vec3 pnoise_permute(vec3 x) {return pnoise_mod289(((x*34.0)+1.0)*x);}
float pnoise_simplexnoise(vec2 v)
{
	const vec4 C = vec4(0.211324865405187, // (3.0-sqrt(3.0))/6.0
	0.366025403784439, // 0.5*(sqrt(3.0)-1.0)
	-0.577350269189626, // -1.0 + 2.0 * C.x
	0.024390243902439); // 1.0 / 41.0
	// First corner
	vec2 i = floor(v + dot(v, C.yy) );
	vec2 x0 = v - i + dot(i, C.xx);

	// Other corners
	vec2 i1;
	//i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
	//i1.y = 1.0 - i1.x;
	i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
	// x0 = x0 - 0.0 + 0.0 * C.xx ;
	// x1 = x0 - i1 + 1.0 * C.xx ;
	// x2 = x0 - 1.0 + 2.0 * C.xx ;
	vec4 x12 = x0.xyxy + C.xxzz;
	x12.xy -= i1;

	// Permutations
	i = pnoise_mod289(i); // Avoid truncation effects in permutation
	vec3 p = pnoise_permute( pnoise_permute( i.y + vec3(0.0, i1.y, 1.0 )) + i.x + vec3(0.0, i1.x, 1.0 ));

	vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
	m = m*m ;
	m = m*m ;

	// Gradients: 41 points uniformly over a line, mapped onto a diamond.
	// The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)

	vec3 x = 2.0 * fract(p * C.www) - 1.0;
	vec3 h = abs(x) - 0.5;
	vec3 ox = floor(x + 0.5);
	vec3 a0 = x - ox;

	// Normalise gradients implicitly by scaling m
	// Approximation of: m *= inversesqrt( a0*a0 + h*h );
	m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );

	// Compute final noise value at P
	vec3 g;
	g.x = a0.x * x0.x + h.x * x0.y;
	g.yz = a0.yz * x12.xz + h.yz * x12.yw;
	return 130.0 * dot(m, g);
}
float pnoise(float x, float y)
{
	vec2 co = vec2(x,y);
	return pnoise_simplexnoise(co);
}