//
// 3.30 GLSL functions only
//
#version 330 core

// A Better Noise Generation
//
// See: http://stackoverflow.com/questions/4200224/random-noise-functions-for-glsl
//
// "The results are very good: it looks exactly like static with every input I tried, no visible patterns at all.
// In contrast the popular sin/fract snippet has fairly pronounced diagonal lines on my GPU given the same inputs."
//
// A single iteration of Bob Jenkins' One-At-A-Time hashing algorithm.
uint hash( uint x )
{
    x += ( x << 10u );
    x ^= ( x >>  6u );
    x += ( x <<  3u );
    x ^= ( x >> 11u );
    x += ( x << 15u );
    return x;
}
// Compound versions of the hashing algorithm I whipped together.
uint hash( uvec2 v ) { return hash( v.x ^ hash(v.y)                         ); }
uint hash( uvec3 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z)             ); }
uint hash( uvec4 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z) ^ hash(v.w) ); }
// Construct a float with half-open range [0:1] using low 23 bits.
// All zeroes yields 0.0, all ones yields the next smallest representable value below 1.0.
float floatConstruct( uint m )
{
    const uint ieeeMantissa = 0x007FFFFFu; // binary32 mantissa bitmask
    const uint ieeeOne      = 0x3F800000u; // 1.0 in IEEE binary32

    m &= ieeeMantissa;                     // Keep only mantissa bits (fractional part)
    m |= ieeeOne;                          // Add fractional part to 1.0

    float  f = uintBitsToFloat( m );       // Range [1:2]
    return f - 1.0;                        // Range [0:1]
}
// Pseudo-random value in half-open range [0:1].
float rand( float x ) { return floatConstruct(hash(floatBitsToUint(x))); }
float rand( vec2  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
float rand( vec3  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
float rand( vec4  v ) { return floatConstruct(hash(floatBitsToUint(v))); }
//
// Psykinematix Interface: unoise(float x, float seed, float granularity) 
// Specifications: return uniform noise between -1 and 1 with seed s (as integer), granularity g (in pixels)
// x arg is only there for compatibility purpose with Texture mode
float unoise(float x, float seed, float granularity)
{
	return 2.*rand(float(granularity)*floor(coordVar/granularity)+seed)-1.; // [-1,1] range
}