//
// Standard Functions
//
float round(float x)
{ return sign(x)*floor(abs(x)+0.5); }
//
// Spatial Dithering
//
float Bayerdithering2(vec2 xy)
{
	mat2 dithermatrix = mat2(1.0,3.0,4.0,2.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering4(vec2 xy)
{
	mat4 dithermatrix = mat4(1.0,9.0,3.0,11.0,15.0,5.0,13.0,7.0,4.0,12.0,2.0,10.0,14.0,8.0,16.0,6.0);
	float limit = dithermatrix[int(xy.x)][int(xy.y)];
	return limit;
}
float Bayerdithering8(vec2 xy)
{
	float limit;
	int x = int(xy.x);
	int y = int(xy.y);
	
	mat4 dithermatrix1 = mat4(1.0,33.0,57.0,9.0,49.0,17.0,25.0,41.0,47.0,31.0,21.0,45.0,15.0,63.0,37.0,5.0);
	mat4 dithermatrix2 = mat4(3.0,35.0,43.0,11.0,51.0,19.0,27.0,59.0,53.0,29.0,23.0,39.0,13.0,61.0,55.0,7.0);
	mat4 dithermatrix3 = mat4(4.0,52.0,60.0,12.0,36.0,20.0,28.0,44.0,46.0,30.0,24.0,56.0,14.0,62.0,40.0,8.0);
	mat4 dithermatrix4 = mat4(2.0,50.0,42.0,10.0,34.0,18.0,26.0,58.0,48.0,32.0,22.0,54.0,16.0,64.0,38.0,6.0);

	if(x<4) // dithermatrix1 or dithermatrix3
	{
		if(y<4) limit = dithermatrix1[x][y];
		else limit = dithermatrix3[x][y-4];
	}
	else // dithermatrix2 or dithermatrix4
	{
		if(y<4) limit = dithermatrix2[x-4][y];
		else limit = dithermatrix4[x-4][y-4];
	}
	return limit;
}
//
// GRATING STIMULUS
// Assume:
// envor, phase in radians
float EnvelopeElement(	float x, float y, float r, float sizeradius, float sigma, float envor, float envratio, float hardedge, float circularshape ) 
{
	float env;
	float sinorad = sin(envor);
	float cosorad = cos(envor);
	float rotatedx = cosorad * x + sinorad * y;
	float rotatedy = sinorad * x - cosorad * y;

	if(hardedge==1.) // Hard-Edge
	{
		if(circularshape==1.)
		{
			if(envratio==1.) env = step(r, sizeradius); // Circular Envelope
			else // Ellipsoidal Envelope
			{
				float u = abs(rotatedx);
				float v = abs(rotatedy);
				float ratiox = (u/sizeradius);
				float ratioy = v/(sizeradius*envratio);
				float insidecheck = ratiox*ratiox+ratioy*ratioy-1.;
				env = step(insidecheck, 0.);
			}
		}
		else // Rectangular Envelope
		{
			float u = abs(rotatedx);
			float v = abs(rotatedy);
			env = step(u,sizeradius)*step(v,sizeradius*envratio);
		}
	}
	else if(hardedge==0.) // Gaussian Envelope
	{
		float denum = 2.*sigma*sigma;
		float denumy = denum*envratio*envratio;

		if(circularshape==1.) // Circular (Gauss(x)*Gauss(y))
		{
			if(sizeradius==0.)	// Pure Gaussian Envelope (ie no plateau)
			{
				if(envratio==1.) env = exp(-r*r/denum); // Circular Gaussian Envelope 
				else // Elongated Gaussian Envelope
				{
					env = exp(-rotatedx*rotatedx/denum-rotatedy*rotatedy/denumy);
				}
			}
			else // Plateau with Gaussian border
			{
				if(envratio==1.)  // Circular Gaussian Envelope
				{
					float theradius = r-sizeradius;	
					if(theradius<0.) env = 1.;	// Plateau (exp(0) == 1.)
					else env = exp(-theradius*theradius/denum); // Gaussian border
				}
				else // "Elliptic" Gaussian Envelope
				{
					float u = abs(rotatedx);
					float v = abs(rotatedy);
					float ratiox = u/sizeradius;
					float ratioy = v/(sizeradius*envratio);
					float ratioxy2 = ratiox*ratiox+ratioy*ratioy;
					float insidecheck = ratioxy2-1.;

					if(insidecheck<0.) env = 1.; // Plateau (exp(0) == 1.)
					else // We have to estimate the distance to the ellipse border
					{
						float uv2 = sqrt(u*u+v*v);
						float distanceonellipse = uv2/sqrt(ratioxy2);
						float distancetoellipse = abs(uv2-distanceonellipse);
						env = exp(-distancetoellipse*distancetoellipse/denum); // Gaussian border
					}
				}
			}
		}
		else // Rectangular Envelope (Gauss(x)+Gauss(y)) Not well defined, to be decided later on
		{
			env = exp(-(rotatedx*rotatedx/denum)-(rotatedy*rotatedy/denumy));
		}
	}
	return env;
}

// Assume:
// caror, envor, phase in radians
vec2 GratingElement( 	float x, float y, float r,
						float sf, float caror, float phase,
						float sizeradius, float sigma, float envor, float envratio, 
						float squarewave, float hardedge, float circularshape )
{
	// Rotation
	float sinorad = sin(caror+pi/2.);
	float cosorad = cos(caror+pi/2.);
	float rotatedx = cosorad * x + sinorad * y;
	
	// Carrier Modulation
	float sfrad = 2.*pi*sf;
	float carrier = cos(sfrad*rotatedx+phase);
	if(squarewave==1.) carrier = sign(carrier);
	
	// Envelop
	float envelop;
	float alpha;
	if(hardedge==-1.) // Fullscreen
	{
		alpha = 1.;
		envelop = carrier;
	}
	else
	{
		alpha = EnvelopeElement(x, y, r, sizeradius, sigma, envor, envratio, hardedge, circularshape);
		envelop = carrier*alpha;
	}

	return vec2(envelop, alpha);
}

// Assume:
// envor, phase in radians
vec2 RadialAngularElement( float x, float y, float r, float theta,
						float sf, float phase,
						float sizeradius, float sigma, float envor, float envratio, 
						float squarewave, float hardedge, float circularshape, float radial )
{	
	// Carrier Modulation
	float sfrad;
	float carrier;
	if(radial==0.) // Angular modulation
	{
		sfrad = round(sf);
		carrier = cos(sfrad*theta+phase);
	}
	else			// Radial modulation
	{
		sfrad = 2.*pi*sf;
		carrier = cos(sfrad*r+phase);
	}
	if(squarewave==1.) carrier = sign(carrier);
	
	// Envelop
	float envelop;
	float alpha;
	if(hardedge==-1.) // Fullscreen
	{
		alpha = 1.;
		envelop = carrier;
	}
	else
	{
		alpha = EnvelopeElement(x, y, r, sizeradius, sigma, envor, envratio, hardedge, circularshape);
		envelop = carrier*alpha;
	}

	return vec2(envelop, alpha);
}

//
// Noise Functions
//
// Differences with original version: addition of seed parameter
float unoise_rand(vec2 co, float seed) // returns a pseudo-random number in [0,1] range 
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * (43758.5453+seed));
	// original without see arg: return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}
// Not used: float noise(float x, float y, float seed) { vec2 co = vec2(x,y); return rand(co, seed); }
//
//	Uniform Noise:	unoise(x, seed, granularity)
//
// Note: using sampled x to the seed prevent the occurences of diagonal patterns with Intel HD Graphics 4000
float unoise(float x, float seed, float granularity) // Psykinematix specification
{
	vec2 sampling = float(granularity)*floor(coordVar/granularity);
	float xseed = float(granularity)*floor(x/granularity);
	return 2.*unoise_rand(sampling,seed+xseed)-1.; // [-1,1] range
	// return rand(fract(coordVar*float(granularity))/size);
}
// 1D version
float unoiseoned(float x, float seed, float granularity) // Psykinematix specification
{
	float sampling = float(granularity)*floor(coordVar.x/granularity);
	float xseed = float(granularity)*floor(x/granularity);
	return 2.*unoise_rand(vec2(sampling,0),seed+xseed)-1.; // [-1,1] range
}
// Note: addition of seed parameter required
vec2 NoiseElement( float x, float y, float r,
						float uniformn, float dim, float color,
						float sizeradius, float sigma, float envor, float envratio, 
						float hardedge, float circularshape, float seed )
{
	// Noise Generation
	float carrier;
	if(dim==1.) // 1D Noise
	{
		if(color==1.)
		{
			// Not supported
		}
		else if(uniformn==0.) // Binary Noise
		{
			carrier = sign(unoiseoned(x, seed, 1.));
		}
		else if(uniformn==1.) // Uniform White Noise
		{
			carrier = unoiseoned(x, seed, 1.);
		}
		else // if(uniformn==2.) // Gaussian Noise
		{
			// Not supported
		}
	}
	else	// 2D Noise
	{
		if(color==1.)
		{
			// Not supported
		}
		else if(uniformn==0.) // Binary Noise
		{
			carrier = sign(unoise(x, seed, 1.));
		}
		else if(uniformn==1.) // Uniform White Noise
		{
			carrier = unoise(x, seed, 1.);
		}
		else // if(uniformn==2.) // Gaussian Noise
		{
			// Not supported
		}
	}

	// Envelop
	float envelop;
	float alpha;
	if(hardedge==-1.) // Fullscreen
	{
		alpha = 1.;
		envelop = carrier;
	}
	else
	{
		alpha = EnvelopeElement(x, y, r, sizeradius, sigma, envor, envratio, hardedge, circularshape);
		envelop = carrier*alpha;
	}

	return vec2(envelop, alpha);
}
						
